﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HEAL.Attic;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {
  [Item("GQAPStochasticSolutionCreator", "Base class for stochastic solution creators of the Generalized Quadratic Assignment Problem.")]
  [StorableType("D7789768-032F-4EAB-ADED-FF4DD5A23B99")]
  public abstract class GQAPStochasticSolutionCreator : GQAPSolutionCreator, IStochasticOperator {

    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }

    [StorableConstructor]
    protected GQAPStochasticSolutionCreator(StorableConstructorFlag _) : base(_) { }
    protected GQAPStochasticSolutionCreator(GQAPStochasticSolutionCreator original, Cloner cloner)
      : base(original, cloner) { }
    public GQAPStochasticSolutionCreator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random number generator to use."));
    }

    protected sealed override IntegerVector CreateSolution(GQAPInstance problemInstance) {
      return CreateRandomSolution(RandomParameter.ActualValue, problemInstance);
    }

    protected abstract IntegerVector CreateRandomSolution(IRandom random, GQAPInstance problemInstance);
  }
}
